%% Phase transformation - calcite - aragonite
%
%  This is a basic script to calculate and plot interphase misorientations
%  and is the basic manuscript used all over the manuscript submitted to
%  the American Mineralogist, the only changes are the input files and the
%  orientation relationships between the different studied phases. The
%  input file for this specific map is also given in the supplementary
%  material

%% Close all figures and clean up workspace

close all
clear all

%% Make a log of the command window

diary calcite_aragonite_example.txt

%% Setup setMTEXpref

setMTEXpref('FontSize',25); %general font size
setMTEXpref('pfAnnotations',@(varargin) 1); % switch off automatic annotations
setMTEXpref('defaultColorMap',WhiteJetColorMap); %default color map

%% Specify Crystal and Specimen Symmetries

CS = {... 
  'notIndexed',...
  crystalSymmetry('321', [4.99 4.99 17.064], 'X||a', 'Y||b*', 'Z||c', 'mineral', 'calcite', 'color', 'light blue'),...
  crystalSymmetry('222', [4.962 7.97 5.739], 'mineral', 'aragonite', 'color', 'light green')};

setMTEXpref('xAxisDirection','east');
setMTEXpref('zAxisDirection','intoPlane');

%% Specify File Names

% path to files, change accordingly to where you stored your files
pname = '/Users/lfgmorales/Desktop/Work in progress/interphase_misorientation/aragonite-calcite';

% which files to be imported
fname = [pname '/010SC_3_X_NPAR.ang'];

%% Import the Data

ebsd = EBSD.load(fname,CS,'interface','ang',...
  'convertEuler2SpatialReferenceFrame','setting 2');

% this input file comes from EDAX-TSL software, if you have used an Oxford
% system to make your measurements, this part here will be different, to I
% suggest you use the import_wizard command for you to import the correct
% file head (Specify Crystal and Specimen Symmetries, paths and the correct
% data format)

%% Create variable with unchanged ebsd data

ebsd2=ebsd; % for original IQ map 
[grains2,ebsd2.grainId,ebsd2.mis2mean] = calcGrains(ebsd2);
ebsd2=ebsd2(grains2);

%% Gridify the data

ebsd = ebsd.gridify; % change the data from hexagonal to regular grid
ebsd(isnan(ebsd.rotations)).phaseId = -1;

%% Calculate grains, discard all grains as below

[grains, ebsd.grainId, ebsd.mis2mean]=calcGrains(ebsd);
ebsd=ebsd(grains(grains.grainSize>10)); %discard all small grains
[grains, ebsd.grainId, ebsd.mis2mean]=calcGrains(ebsd,'angle',10*degree,'unitcell');
ebsd=ebsd(grains)

%% Plot image quality and phase map together

close all
plot(ebsd2,ebsd2.iq,'figSize','large'); 
hold on
legend off
mtexColorMap gray
plot(grains,'FaceAlpha',0.3)
hold off

%% Plot orientation map calcite

ipfKey=ipfHSVKey(ebsd('c'));
plot(ipfKey)

color=ipfKey.orientation2color(ebsd('c').orientations);
plot(ebsd('c'),color);
hold on
plot(grains.boundary);
hold off

%% Plot mis2mean

plot(ebsd('c'),ebsd('c').mis2mean.angle ./ degree,'figSize','large')
hold on
plot(grains.boundary('c'))
hold off
caxis([0 10])
mtexColorbar

%% Grains boundary calcite and aragonite, and interphase boundary

gB_c_c=grains.boundary('c','c');
gB_c_a=grains.boundary('c','a');
gB_a_a=grains.boundary('a','a');
gB=grains.boundary

%% ODF calcite and aragonite

odf_c=calcDensity(ebsd('c').orientations,'halfwidth',10*degree);
odf_a=calcDensity(ebsd('a').orientations,'halfwidth',10*degree);

%% Pole figures definition short

PFs_c_hkil_short = [ ...
    Miller(0,0,0,1,ebsd('c').CS,'hkil','phase','c'), ...
    Miller(2,-1,-1,0,ebsd('c').CS,'hkil','phase','c'),...
    Miller(1,0,-1,4,ebsd('c').CS,'hkil','phase','c'),...
    Miller(0,1,-1,8,ebsd('c').CS,'hkil','phase','c'),...
    ];

PFs_a_uvw_short = [ ...
    Miller(1,0,0,ebsd('a').CS,'uvw','phase','a'), ...
    Miller(0,1,0,ebsd('a').CS,'uvw','phase','a'),...
    Miller(0,0,1,ebsd('a').CS,'uvw','phase','a'),...
    Miller(1,1,0,ebsd('a').CS,'uvw','phase','a'),...
    ];

%% Pole figures calcite-aragonite short
figure (1)
setMTEXpref('FontSize',25)
plotPDF(odf_c,PFs_c_hkil_short,'antipodal','resolution',5*degree,'contourf','minmax');
CLim(gcm,'equal');
mtexColorbar 
annotate([xvector,yvector,-zvector],'label',{'X','Z','Y'},'backgroundcolor','w','MarkerSize',20,'FontSize',20)
drawNow(gcm,'figSize','medium')
saveFigure('010SC_3_X_calcite_pole_figures_short.png')

figure (2)
setMTEXpref('FontSize',25)
plotPDF(odf_a,PFs_a_uvw_short,'antipodal','resolution',5*degree,'contourf','minmax');
CLim(gcm,'equal');
mtexColorbar 
annotate([xvector,yvector,-zvector],'label',{'X','Z','Y'},'backgroundcolor','w','MarkerSize',20,'FontSize',20)
drawNow(gcm,'figSize','medium')
saveFigure('010SC_3_X_aragonite_pole_figures_short.png')
%% MDFs interphase boundaries

MDF_c_a = calcMDF(odf_c,odf_a);

%% Interphase misorientation angle distribution

close all
figure(1)
nbins=50;
plotAngleDistribution(gB_c_a.misorientation,nbins,'FaceColor','r','EdgeColor','k')
hold all 
plotAngleDistribution(MDF_c_a)
plotAngleDistribution(ebsd('c').CS,ebsd('a').CS,'FontSize',18)
set(gca,'FontSize',20)
hold off
% plot title
title('Misorientation angles distribution: Calcite - Aragonite','FontSize',18)
drawNow(gcm,'figSize','large')
legend({'correlated','uncorrelated','uniform'},'location','NorthWest','FontSize',18)

%% Angles & axes - Calcite and Aragonite - uniform distribution

[density,omega] = calcAngleDistribution(ebsd('c').CS,ebsd('a').CS);
omega_deg = omega./degree;
frequency_percent = 100 * density ./sum(density);
[max_frequency findex]  = max(frequency_percent);
omega_at_max_ol_atg = omega_deg(findex)
frequency_at_max_plg_cpx = frequency_percent(findex)
close all
figure
plot(omega_deg,frequency_percent,'b','LineWidth',2)
axis tight
xlabel('Misorientation angle in degrees','FontSize',18)
ylabel('Frequency in percent','FontSize',18)
title('Uniform misorientation angle distribution: Calcite - Aragonite','FontSize',18)

%% Calculation of maximum from uniform MDF interphase - calcite - aragonite (to annotate in the next part)

max_MDF_c_a=calcAxisDistribution(MDF_c_a)
[~, m_MDF_c_a] = max(max_MDF_c_a)
round(m_MDF_c_a)
ma_MDF_c_a=Miller(vector3d(m_MDF_c_a),ebsd('c').CS);
ma_MDF_c_a_sym=symmetrise(ma_MDF_c_a)
ma_MDF_c_a_sym=round(ma_MDF_c_a_sym,'maxHKL',6) % option to round with maximum hkl = 6

%% Uniform misorientation axis calcite-aragonite
close all
figure (1)
plotAxisDistribution(ebsd('c').CS,ebsd('a').CS,'fundamentalRegion')
mtexColorbar
annotate([Miller(-2,1,1,0,ebsd('c').CS,'hkl'),...
          Miller(1,-2,1,0,ebsd('c').CS,'hkl'),...
          Miller(1,1,-2,0,ebsd('c').CS,'hkl'),...
          Miller(0,0,0,1,ebsd('c').CS,'hkl')],...
          Miller(1,0,-1,0,ebsd('c').CS,'hkl'),...
          'all','labeled','BackgroundColor','w','FontSize',18,'MarkerSize',20)
      hold on
      cs=ebsd('c').CS
h = Miller({1,0,-1,0},{1,1,-2,0},{1,0,-1,4},{0,1,-1,8},{0,0,0,1},cs);
h_s=symmetrise(h)
annotate(h_s,'all','labeled','BackgroundColor','w','FontSize',18,'MarkerSize',20)
annotate(ma_MDF_c_a_sym,'all','labeled','BackgroundColor','w','FontSize',18,'MarkerSize',20)
hold off
drawNow(gcm,'figSize','large')

%% Plot misorientation axis distribution calcite-aragonite correlated

plotAxisDistribution(gB_c_a.misorientation,'contourf','fundamentalRegion')
mtexColorbar
annotate([Miller(-2,1,1,0,ebsd('c').CS,'hkl'),...
          Miller(1,-2,1,0,ebsd('c').CS,'hkl'),...
          Miller(1,1,-2,0,ebsd('c').CS,'hkl'),...
          Miller(0,0,0,1,ebsd('c').CS,'hkl')],...
          Miller(1,0,-1,0,ebsd('c').CS,'hkl'),...
          'all','labeled','BackgroundColor','w','FontSize',18,'MarkerSize',20)
      hold on
      cs_c=ebsd('c').CS
      cs_a=ebsd('a').CS
h = Miller({1,0,-1,0},{2,-1,-1,0},{1,1,-2,0},{1,0,-1,4},{0,1,-1,8},{0,0,0,1},cs);
h_s=symmetrise(h)
annotate(h_s,'all','labeled','BackgroundColor','w','FontSize',18,'MarkerSize',20)
hold off
drawNow(gcm,'figSize','large')

%% Calculation of maximum from uniform MDF interphase - calcite - aragonite (to annotate in the next part)

gB_c_a_mis = grains.boundary('c','a').misorientation;
gb_c_a_axis_cor = axis(gB_c_a_mis.project2FundamentalRegion);
gb_c_a_angle_cor = angle(gB_c_a_mis.project2FundamentalRegion)./ degree;

lower_limit = 25  
upper_limit = 50
select_angle_range = gb_c_a_angle_cor >= lower_limit & gb_c_a_angle_cor < upper_limit;
selected_angles = gb_c_a_angle_cor(select_angle_range);
selected_axes = gb_c_a_axis_cor(select_angle_range);
selected_axes_d=calcDensity(selected_axes,'halfwidth',5*degree)% density of selected axes - S2FunHarmonic
selected_axes_u=max_MDF_c_a % density of uniform MDF for specific phase boundary - S2FunHarmonic
selected_axes_n=selected_axes_d./selected_axes_u % selected axes normalized against uniform
[mpos, mval] = selected_axes_d.max('numlocal',1)
round(mval)

ma_mval_c=Miller(vector3d(mval),ebsd('c').CS);
ma_mval_c_sym=symmetrise(ma_mval_c)
ma_mval_c_sym=round(ma_mval_c,'maxHKL',6) % option to round with maximum hkl = 6

ma_mval_a=Miller(vector3d(mval),ebsd('a').CS);
ma_mval_a_sym=symmetrise(ma_mval_a)
ma_mval_a_sym=round(ma_mval_a,'maxHKL',6)

close all
figure(1)
plot(selected_axes_n,'contourf','fundamentalRegion','linecolor','b','linewidth',1,'resolution',1*degree)
mtexColorbar
cs_c=ebsd('c').CS
cs_a=ebsd('a').CS
h_c = Miller({1,0,-1,0},{1,1,-2,0},{1,0,-1,4},{0,1,-1,8},{0,0,0,1},cs_c);
h_c_s=symmetrise(h_c)
annotate(h_c_s,'all','labeled','antipodal','BackgroundColor','w','FontSize',18,'MarkerSize',20)
drawNow(gcm,'figSize','large')

figure(2)
plot(selected_axes_n,'contourf','fundamentalRegion','linecolor','b','linewidth',1,'resolution',1*degree)
mtexColorbar
cs_c=ebsd('c').CS
cs_a=ebsd('a').CS
h_a = Miller({1,0,0},{0,1,0},{0,0,1},{1,1,0},{1,1,3},{0,1,1},cs_a);
h_a_s=symmetrise(h_a)
annotate(h_a_s,'all','labeled','antipodal','BackgroundColor','w','FontSize',18,'MarkerSize',20)
drawNow(gcm,'figSize','large')

%% Create search structure for boundary lengths per misorientation angle interval
% N.B. With this method you select using uniquely the misorientation angle
%      so if you have several different misorienation axes in a given
%      angle interval they will be added together
% minimum:interval:maximum with interval = 5,10,15... etc try several

gb_m_axis_cor = axis(gB_c_a_mis.project2FundamentalRegion);
gb_m_angle_cor = angle(gB_c_a_mis.project2FundamentalRegion)./ degree;
min_angle = 10;
interval_angle = 10;
maximum_angle = 120;
[segments_per_interval,interval_id] = histc(gb_m_angle_cor,min_angle:interval_angle:maximum_angle);
x_bar=min_angle:interval_angle:maximum_angle;
width=1;
clear percent_boundary_length n_percent angle_scale;
percent_boundary_length = zeros(length(segments_per_interval));
% total boundary length
total_boundary_length = sum(gB_c_a.segLength);
% total number
total_number = sum(segments_per_interval);
%
fprintf(' \n')
fprintf(' Boundary lengths per misorientation angle interval \n')
fprintf(' \n')
fprintf(' %s %s  \n','Boundary lengths in',gB_c_a.scanUnit)
fprintf(' \n')
fprintf('  ID  Interval         gB-Length   gB-Percent          n-Counts    n-Percent \n')
for i=1:length(segments_per_interval)-1;
% calculate boundary length in map units
boundary_length = sum(gB_c_a(interval_id==i).segLength);
% Percent boundary length
percent_boundary_length(i,1) = 100 * boundary_length / total_boundary_length;
% Percent number
n_percent(i,1) = 100 * segments_per_interval(i) / total_number;
    fprintf(' %3i %3i %s %3i %17.2f %12.2f %17.2f %12.2f \n',...
      i,x_bar(i),'-',x_bar(i+1),boundary_length,percent_boundary_length(i),segments_per_interval(i),n_percent(i))
  angle_scale(i,1) = x_bar(i);
end

%% Plot selected grains with misorientation boundaries of 20-60 degrees

plotx2east
close all
plot(ebsd)
hold on
plot(grains('c').boundary,'linecolor','k','linewidth',1)
plot(grains('a').boundary,'linecolor','k','linewidth',1)
plot(gB_c_a(interval_id==2),'linecolor','r','linewidth',2,'displayName','20-30^\circ 8%','figSize','large')
plot(gB_c_a(interval_id==3),'linecolor','m','linewidth',2,'displayName','30-40^\circ 12%','figSize','large')
plot(gB_c_a(interval_id==4),'linecolor','c','linewidth',2,'displayName','40-50^\circ 14%','figSize','large')
plot(gB_c_a(interval_id==5),'linecolor','k','linewidth',2,'displayName','50-60^\circ 15%','figSize','large')
hold off
legend({},'location','NorthWest','FontSize',16)
set(gcf,'renderer','zbuffer')
drawNow(gcm,'figSize','large')

%% Calculate the expect orientation of parent phase
%
%  If know the orientation of the daugher phase and the orientation
%  relationship between parent-daughter phases, we can calculate the
%  orientation of the parent phase
%
% Define individual crystal systems

cs_cal = crystalSymmetry('321', [4.99 4.99 17.064], 'X||a', 'Y||b*',...
    'Z||c', 'mineral', 'calcite', 'color', 'light blue');
cs_ara = crystalSymmetry('222', [4.962 7.97 5.739], 'mineral',...
    'aragonite', 'color', 'light green');

% Orientation of daughter phase

ori_aragonite=discreteSample(ebsd('a').orientations,1000)

% Define the misorientation based on the growth relation

u1 = Miller(1,1,-2,0,cs_cal)
u2 = Miller(0,0,0,1,cs_cal,'uvw')
v1 = Miller(1,0,0,cs_ara)
v2 = Miller(0,1,1,cs_ara,'uvw')
mori_c_a_rel = orientation.map(u1,v1,u2,v2,cs_cal,cs_ara);

% Original calcite orientation

o_cal_original=ori_aragonite * symmetrise(mori_c_a_rel,'unique')

% Calculate ODF for the original calcite orientation

odf_cal_original=calcDensity(o_cal_original,'halfwidth',10*degree);

% Plot pole figures

figure (1)
setMTEXpref('FontSize',25)
plotPDF(odf_cal_original,PFs_c_hkil_short,'antipodal','resolution',5*degree,'contourf','minmax');
CLim(gcm,'equal');
mtexColorbar 
annotate([xvector,yvector,-zvector],'label',{'X','Z','Y'},'backgroundcolor','w','MarkerSize',20,'FontSize',20)
drawNow(gcm,'figSize','medium')


%% Orientation relationship
%      Parent           Daughter
% (11-20) Calcite || (100) Aragonite
% [0001] Calcite || [011] Aragonite

% Given in Gillet et al., (1987) - Bulletin Mineralogie

Cal2Ara_No1 = orientation('map',...
  Miller(1,1,-2,0,ebsd('calcite').CS,'hkl'),Miller(1,0,0,ebsd('aragonite').CS,'hkl'),...
  Miller(0,0,0,1,ebsd('calcite').CS,'uvw'),Miller(0,1,1,ebsd('aragonite').CS,'uvw'))
%
% check angles between hkl and uvw of parent and daughter to be 90 degrees
angle_hkl_uvw_parent = angle(Miller(1,1,-2,0,ebsd('calcite').CS,'hkl'),...
                             Miller(0,0,0,1,ebsd('calcite').CS,'uvw'))/degree
angle_hkl_uvw_daughter = angle(Miller(1,0,0,ebsd('aragonite').CS,'hkl')...
                             ,Miller(0,1,1,ebsd('aragonite').CS,'uvw'))/degree


% In the code above, ‘map’ refers to any EBSD map which contains the 
% two phases of interest. From these definitions, one can calculate the 
% misorientation angle and misorientation axes and angles expected from 
% this orientation relationship

% axis vector3d
axis_No1 = vector3d(axis(Cal2Ara_No1))
% axis Aragonite
axis_No1_Ara = axis(Cal2Ara_No1)
% axis Calcite
axis_No1_Cal = Miller(axis_No1,ebsd('calcite').CS,'hkl')
% rotation angle
angle_No1 = angle(Cal2Ara_No1)/degree
% Euler angles
f1Ff2_No1 = Euler(Cal2Ara_No1)/degree
% Calcite reference orientation Euler (0,0,0)
ori_cal = orientation('Euler',0,0,0,ebsd('calcite').CS)
% Calculate all Daughter (Aragonite) variants 
% by using all crystalloraphically equivalent orientations
% of Parent (Calcite)
ori_Ara_No1 = symmetrise(ori_cal) * inv(Cal2Ara_No1)

